#pragma once

#include <common/common.h>
#include <pthread.h>
#include <sys/types.h>

struct poller_thread {
    /* The input poll. */
    volatile unsigned int input_len;
    volatile unsigned int input_size;
    volatile void *volatile *volatile input_cells;
    pthread_mutex_t input_lock;

    pthread_t th;
    unsigned int initial_size;

    /* The output pool. */
    volatile unsigned int output_len;
    volatile unsigned int output_size;
    volatile void *volatile *volatile output_cells;
    pthread_mutex_t output_lock;
};

struct poller_thread_arg {
    struct poller_thread *self;
    void *args;
};


/* Create a new thread. Returns 0 on success, a non zero value on error. */
int poller_new(struct poller_thread *th, unsigned int init_sizes, void *(*func)(struct poller_thread_arg *), void *args);

/* Joins a thread. Returns 0 on success, a non zero value on error. */
int poller_join(struct poller_thread *th, void **ret);

/* Input qeueu manipulations. */
int poller_append_input(struct poller_thread *th, void *ptr);
int poller_pop_input   (struct poller_thread *th, void **ptr);
int poller_head_input  (struct poller_thread *th, void **ptr);

/* Output qeueu manipulations. */
int poller_append_output(struct poller_thread *th, void *ptr);
int poller_pop_output   (struct poller_thread *th, void **ptr);
int poller_head_output  (struct poller_thread *th, void **ptr);
