#pragma once

#define TWO_LEVEL

#include <stdlib.h>
#include <stdio.h>
#include <ucontext.h>

#include <mthread/mthread.h>
#include <common/common.h>

#define MTHREAD_DEFAULT_STACK           BRACKETS_THAT(128*1024) /* 128 kB */
#define MTHREAD_MAX_VIRUTAL_PROCESSORS  256

struct mthread_list_s {
    volatile struct mthread_s *first;
    volatile struct mthread_s *last;
    mthread_tst_t lock;
};

typedef struct {
    struct mthread_s *idle;
    volatile struct mthread_s *current;
    mthread_list_t ready_list;
    int rank;
    volatile int state;
    volatile struct mthread_s *resched;
    volatile mthread_tst_t *p;
} mthread_virtual_processor_t;

typedef enum { RUNNING, BLOCKED, ZOMBIE } mthread_status_t;
typedef volatile enum { JOINABLE = 0, DETACHED = 1, DETACHED_FREE = 2 } mthread_detached_flag_t;

#define INIT_KEYS_LIST 48

struct keys_list {
    volatile mthread_key_t *list;
    unsigned int first_avail;
    unsigned int size;
};

struct mthread_s {
    ucontext_t uc;
    volatile void *res;
    void *arg;
    void *(*__start_routine) (void *);
    volatile struct mthread_s *next;
    volatile mthread_status_t status;
    mthread_detached_flag_t detached;
    int not_migrable;
    mthread_virtual_processor_t *vp;
    void *stack;
    struct keys_list keys;
};

#define MTHREAD_LIST_INIT { .first = NULL, .last = NULL, .lock = 0 }

int  mthread_test_and_set   (mthread_tst_t *atomic);
void mthread_spinlock_lock  (mthread_tst_t *atomic);
void mthread_spinlock_unlock(mthread_tst_t *atomic);
int  mthread_get_vp_rank();

void mthread_insert_first(struct mthread_s *item, mthread_list_t *list);
void mthread_insert_last (struct mthread_s *item, mthread_list_t *list);
int  mthread_list_test   (struct mthread_s *item, mthread_list_t *list);
struct mthread_s *mthread_remove_first(mthread_list_t *list);

void __mthread_yield(mthread_virtual_processor_t *vp);
mthread_virtual_processor_t *mthread_get_vp();
