#define _POSIX_C_SOURCE 200809L

#include <common/common.h>
#include <lektor/database.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <stdio.h>

static inline int
__check_type(const char *type)
{
    return ! ( STR_MATCH(type, "kara") || STR_MATCH(type, "plt") );
}

bool
database_sticker_create(volatile sqlite3 *db, const char *name)
{
    static const char *INSERT =
        "INSERT INTO 'stickers' (id, name)"
        " SELECT CASE WHEN MAX(id) IS NULL THEN 1 ELSE MAX(id) + 1 END, ?"
        " FROM 'stickers'"
        "WHERE (SELECT COUNT(*) FROM 'stickers' WHERE name = ?) = 0;";
    sqlite3_stmt *stmt;
    int ret = false;
    RETURN_IF(strlen(name) == 0, "A sticker name must be at least one character long", ret);
    SQLITE_PREPARE(db, stmt, INSERT, error);
    SQLITE_BIND_TEXT(db, stmt, 1, name, error);
    SQLITE_BIND_TEXT(db, stmt, 2, name, error);
    SQLITE_STEP_DONE(db, stmt, error);
    ret = true;
error:
    sqlite3_finalize(stmt);
    return ret;
}

bool
database_sticker_delete(volatile sqlite3 *db, const char *name)
{
    static const char *INSERT = "DELETE FROM 'stickers' WHERE name = ?;";
    sqlite3_stmt *stmt;
    int ret = false;
    RETURN_IF(strlen(name) == 0, "A sticker name must be at least one character long", ret);
    SQLITE_PREPARE(db, stmt, INSERT, error);
    SQLITE_BIND_TEXT(db, stmt, 1, name, error);
    SQLITE_STEP_DONE(db, stmt, error);
    ret = true;
error:
    sqlite3_finalize(stmt);
    return ret;
}

bool
database_sticker_set(volatile sqlite3 *db, const char *type, const char *name, int uri, int value)
{
    static const char *SQL_TEMPLATE =
        "INSERT OR REPLACE INTO 'stickers.%s' (id, sticker, value) "
        "SELECT ?, 'stickers'.id, ? "
        "FROM 'stickers' "
        "WHERE 'stickers'.name = ?;";
    char SQL[LKT_MAX_SQLITE_STATEMENT];
    sqlite3_stmt *stmt;
    int ret = false;

    if (__check_type(type)) {
        LOG_ERROR_SCT("DB", "Type '%s' is invalid", type);
        return false;
    }

    safe_snprintf(SQL, LKT_MAX_SQLITE_STATEMENT, SQL_TEMPLATE, type);

    /* Bindings:
     * 1 -> the uri of the plalist or kara
     * 2 -> the value of the sticker
     * 3 -> the name of the sticker
     */

    SQLITE_PREPARE(db, stmt, SQL, error);
    SQLITE_BIND_INT(db, stmt, 1, uri, error);
    SQLITE_BIND_INT(db, stmt, 2, value, error);
    SQLITE_BIND_TEXT(db, stmt, 3, name, error);
    SQLITE_STEP_DONE(db, stmt, error);
    ret = true;
error:
    sqlite3_finalize(stmt);
    return ret;
}

bool
database_sticker_delete_specify(volatile sqlite3 *db, const char *type, int uri, const char *name)
{
    char SQL[LKT_MAX_SQLITE_STATEMENT];
    sqlite3_stmt *stmt;
    int ret = false;
    if (__check_type(type)) {
        LOG_ERROR_SCT("DB", "Type '%s' is invalid", type);
        return false;
    }

    safe_snprintf(SQL, LKT_MAX_SQLITE_STATEMENT, "DELETE FROM 'stickers.%s' "
                  "WHERE 'stickers.%s' = ? ", type, type);

    /* If there is a name specified. */
    if (!name) {
        strncat(SQL, ";", LKT_MAX_SQLITE_STATEMENT - 1);
        SQL[LKT_MAX_SQLITE_STATEMENT - 1] = 0;
    } else {
        strncat(SQL, "AND sticker = (SELECT id FROM 'stickers' WHERE name = ? LIMIT 1);",
                LKT_MAX_SQLITE_STATEMENT - 1);
        SQL[LKT_MAX_SQLITE_STATEMENT - 1] = 0;
    }

    SQLITE_PREPARE(db, stmt, SQL, error);
    SQLITE_BIND_INT(db, stmt, 1, uri, error);

    if (name)
        SQLITE_BIND_TEXT(db, stmt, 2, name, error);

    SQLITE_STEP_DONE(db, stmt, error);
    ret = true;
error:
    sqlite3_finalize(stmt);
    return ret;
}
