#define _POSIX_C_SOURCE 200809L

#include <string.h>
#include <unistd.h>

#include <lektor/bufferfd.h>

static inline ssize_t
__fill(struct bufferfd *bf)
{
    if (bf->len <= bf->pos) {
        ssize_t n = read(bf->fd, bf->buffer, BUFFER_MAX);

        if (n < 0)
            return -1;

        bf->len = (size_t) n;
        bf->pos = 0;
    }

    return bf->len - bf->pos;
}

ssize_t
bufferfd_bytes(struct bufferfd *bf, size_t n, uint8_t *res)
{
    ssize_t r = __fill(bf); /* TODO repeat until n is reached */

    if (r < (ssize_t) n)
        return -1;

    memcpy(res, bf->buffer + bf->pos, n);
    bf->pos += n;
    return 1;
}

ssize_t
bufferfd_byte(struct bufferfd *bf, uint8_t *res)
{
    return bufferfd_bytes(bf, 1, res);
}

static inline ssize_t
__skip(struct bufferfd *bf, const size_t n, const size_t b_size)
{
    uint8_t b[b_size];
    return bufferfd_bytes(bf, n, b);
}

ssize_t
bufferfd_skip(struct bufferfd *bf, const size_t n)
{
    return __skip(bf, n, n > BUFFER_MAX ? BUFFER_MAX : n);
}

ssize_t
bufferfd_seek(struct bufferfd *bf, size_t pos)
{
    if (lseek(bf->fd, pos, SEEK_SET) < 0)
        return -1;

    // The call to lseek invalidated bf->buffer.
    bf->pos = 0;
    bf->len = 0;
    return 0;
}
