#define _POSIX_C_SOURCE 200809L

#include <mthread/mthread_internal.h>

/* Functions for handling semaphore.  */

int
mthread_sem_init(mthread_sem_t *sem, unsigned int value)
{
    sem->value = value;
    mthread_mutex_init(&sem->mutex, NULL);
    return 0;
}

/* P(sem), wait(sem) */
int
mthread_sem_wait(mthread_sem_t *sem)
{
    /* Here we loop while we can't enter the semaphore because there are already
       the maximum number of clients. */
loop:
    mthread_mutex_lock(&sem->mutex);
    if (sem->value == 0) {
        mthread_mutex_unlock(&sem->mutex);
        mthread_yield();
        goto loop;
    }

    sem->value --;
    mthread_mutex_unlock(&sem->mutex);
    return 0;
}

/* V(sem), signal(sem) */
int
mthread_sem_post(mthread_sem_t *sem)
{
    mthread_mutex_lock(&sem->mutex);
    sem->value ++;
    mthread_mutex_unlock(&sem->mutex);
    return 0;
}

int
mthread_sem_getvalue(mthread_sem_t *sem, int *sval)
{
    *sval = sem->value;
    return 0;
}

int
mthread_sem_trywait(mthread_sem_t *sem)
{
    if (mthread_mutex_trylock(&sem->mutex))
        return 1;

    if (sem->value == 0) {
        mthread_mutex_unlock(&sem->mutex);
        return 1;
    }

    sem->value --;
    mthread_mutex_unlock(&sem->mutex);
    return 0;
}

/* undo sem_init() */
int
mthread_sem_destroy(mthread_sem_t *sem)
{
    mthread_mutex_destroy(&sem->mutex);
    sem->value = 0;
    return 0;
}
